-- This module handle various UI Elements by LUI or Blizzard.

------------------------------------------------------
-- / SETUP AND LOCALS / --
------------------------------------------------------
local addonname, LUI = ...
local module = LUI:NewModule("Control Panel")
local L = LUI.L

-- constants
local SUBTITLE_FORMAT = SUBTITLE_FORMAT

-- locals
local modInfotext

-- defaults
module.defaults = {
	profile = {
		--Empty
	},
}
------------------------------------------------------
-- / MODULE FUNCTIONS / --
------------------------------------------------------
--[[
-- Generate an Enable button as seen in the Modules pane.
-- @function devapi:NewEnableButton
-- @param ...
-- @func enableFunc Bool function to determine Enable/Disable state.
-- @func func The function to call when the button is pressed.
-- @param ...
function devapi:NewEnableButton(name, desc, order, enableFunc, func, width, disabled, hidden)
	local function nameFunc()
		return format("%s: %s", name, (enableFunc()) and L["API_BtnEnabled"] or L["API_BtnDisabled"])
	end
	local t = self:NewExecute(nameFunc, desc, order, func, width, disabled, hidden)
	return t
end
 Experiemntal new EnableButton option
local function enableNameFunc(name)
	return format(SUBTITLE_FORMAT, name,
end

function module:CreateEnableButton()
end
--]]
function module:ModuleIterator()
	local args = {}
	for modName, mod in LUI:IterateModules() do
		if mod.enableButton then
			args[modName] = module:NewEnableButton(modName, L["Core_ModuleClickHint"], nil,
				function() return mod:IsEnabled() end, --enableFunc
				function()
					if IsShiftKeyDown() then
						mod.db:ResetProfile()
						mod:ModPrint(L["Core_ModuleReset"])
					else
						mod:Toggle()
						mod:ModPrint( (mod:IsEnabled()) and L["API_BtnEnabled"] or L["API_BtnDisabled"])
					end
				end)
		end
	end
	return args
end

function module:InfotextIterator()
	local args = {}
	for name, dataObj in modInfotext.LDB:DataObjectIterator() do
		args[name] = module:NewEnableButton(name, nil, nil,
			function() return modInfotext:IsInfotextShown(name) end,
			function()
				modInfotext:ToggleInfotext(name)
			end)
	end
	return args
end

------------------------------------------------------
-- / FRAMEWORK FUNCTIONS / --
------------------------------------------------------
module.name = "Control Panel"
module.order = 5

function module:OnInitialize()
	LUI:RegisterModule(module)
	modInfotext = LUI:GetModule("Infotext")
end

function module:OnEnable()
	db = module.db.profile
end

function module:LoadOptions()
	local options = {
		Modules = module:NewGroup(L["CPanel_Modules"], 1, nil, nil, module:ModuleIterator()),
		Infotext = module:NewGroup(L["CPanel_Infotext"], 2, nil, nil, module:InfotextIterator()),
	}
	return options
end