-- Bags.
-- TODO: Move Bank stuff to Bank element.

------------------------------------------------------
-- / SETUP AND LOCALS / --
------------------------------------------------------
local addonname, LUI = ...
local module = LUI:NewModule("Bags", "AceHook-3.0", "AceBucket-3.0")
local Media = LibStub("LibSharedMedia-3.0")
local L = LUI.L
local db

-- local copies
local floor, format, tinsert, tconcat = floor, format, tinsert, table.concat
local GetContainerNumFreeSlots = GetContainerNumFreeSlots
local GetContainerItemCooldown = GetContainerItemCooldown
local SetItemButtonDesaturated = SetItemButtonDesaturated
local GetBackpackCurrencyInfo = GetBackpackCurrencyInfo
local CooldownFrame_SetTimer = CooldownFrame_SetTimer
local GetContainerNumSlots = GetContainerNumSlots
local GetContainerItemInfo = GetContainerItemInfo
local GetContainerItemLink = GetContainerItemLink
local SetItemButtonTexture = SetItemButtonTexture
local GetItemQualityColor = GetItemQualityColor
local SetItemButtonCount = SetItemButtonCount
local IsBattlePayItem = IsBattlePayItem
local GetMoneyString = GetMoneyString
local CloseAllBags = CloseAllBags
local GetItemInfo = GetItemInfo
local C_NewItems = C_NewItems
local GetMoney = GetMoney
local _G = _G

-- constants
local MAX_WATCHED_TOKENS = MAX_WATCHED_TOKENS
local SEARCH = SEARCH

local CURRENCY_FORMAT = "%d\124T%s:0:0:0:0\124t"
local BAG_TEXTURE_SIZE = 30
local LAYOUT_OFFSET = 26
local NUM_BAG_IDS = 5
local BAG_ID_TABLE = { 0, 1, 2, 3, 4, }

--local BANK_ID_TABLE = { -1, 5, 6, 7, 8, 9, 10, 11, }

-- local variables
local LUIBags
local updateBucket
local currencyString = {}

-- Cache tables
local BagsInfo = {} -- Contains info about the bags. (Need better understanding)
local ItemSlots = {} -- Contains the individual frames for the item slots.
local BagsBarSlots = {} -- Contains the individual frames for the BagsBar.

-- defaults
module.defaults = {
	profile = {
		Lock = false,
		Copybags = true,
		--BlackFrameBG = false,
		Bags = {
			X = 500,
			Y = 500,
			RowSize = 12,
			Padding = 8,
			Spacing = 4,
			Scale = 1,
			BagBar = true,
			BagBarScale = 1,
			ItemQuality = false,
			BackgroundTexture = "Blizzard Tooltip",
			BorderTexture = "Stripped_medium",
			BorderSize = 5,
	 	},
		Fonts = {
			Bags = { Name = "AvantGarge_LT_Medium", Size = 12, Flag = "OUTLINE", },
		},
		Colors = {
			Search =      { r = 0.6,  g = 0.6,  b = 1,    a = 1,   t = "Class",      },
			Border =      { r = 0.2,  g = 0.2,  b = 0.2,  a = 1,   t = "Individual", },
			Background =  { r = 0.18, g = 0.18, b = 0.18, a = 0.8, t = "Individual", },
			Professions = { r = 0.1,  g = 0.5,  b = 0.2,                             },
			--FrameBackground = { r = 0.09, g = 0.09, b = 0.09, a = 0.8, t = "Individual", },
		},
	},
}

------------------------------------------------------
-- / HOOK/SCRIPT FUNCTIONS / --
------------------------------------------------------

function module.OpenBags()
	LUIBags:Show()
end

function module.CloseBags()
	LUIBags:Hide()
end

function module.ToggleBags()
	if LUIBags:IsShown() then
		LUIBags:Hide()
	else
		LUIBags:Show()
	end
end

local function Bags_OnShow()
	-- Need more defined functions
	updateBucket = module:RegisterBucketEvent("BAG_UPDATE", 0.2, "BagSlotUpdate")
	module:RegisterEvent("ITEM_LOCK_CHANGED", "ItemLockUpdate")
	module:BagLayout()
end

local function Bags_OnHide()
	module:UnregisterBucket(updateBucket)
	module:UnregisterEvent("ITEM_LOCK_CHANGED")
end

local function Bags_StartMoving()
	if not db.Lock then
		LUIBags:StartMoving()
	end
end

local function Bags_StopMoving()
	LUIBags:StopMovingOrSizing()
	LUIBags:SetUserPlaced(false)

	local x, y = LUIBags:GetCenter()
	db.Bags.X = format("%.1f", x)
	db.Bags.Y = format("%.1f", y)
end

------------------------------------------------------
-- / FRAME HANDLING FUNCTIONS / --
------------------------------------------------------
function module:CreateSearchEditBox()
	
	--TODO: Research how the WoD bags do their search, 
	-- we may be able to combine all three of those into a single frame.
	
	-- Editbox
	local editbox = CreateFrame("EditBox", nil, LUIBags)
	editbox:Hide()
	editbox:SetHeight(32)
	editbox:SetAutoFocus(true)
	
	local function OpenEditbox(self)
		local parent = self:GetParent()
		parent.search:Hide()
		parent.gold:Hide()
		parent.currency:Hide()
		parent.editbox:Show()
		parent.editbox:HighlightText()
	end
	
	local function HideEditbox(self)
		local parent = self:GetParent()
		parent.search:Show()
		parent.gold:Show()
		parent.currency:Show()
		parent.editbox:ClearFocus()
		parent.editbox:Hide()
		--module:SearchReset()
	end
	
	local function UpdateSearch(self, text)
		--if text then module:SearchUpdate(self:GetText()) end
	end
	
	editbox:SetScript("OnTextChanged", UpdateSearch)
	editbox:SetScript("OnEscapePressed", HideEditbox)
	editbox:SetScript("OnEnterPressed", HideEditbox)
	editbox:SetScript("OnEditFocusLost", HideEditBox)
	editbox:SetScript("OnEditFocusGained", editbox.HighlightText)
	editbox:SetText(SEARCH)
	
	-- Search Text
	local search = LUIBags:CreateFontString(nil, "ARTWORK", "GameFontHighlightLarge")
	search:SetPoint("TOPLEFT", LUIBags, db.Bags.Padding, -10)
	search:SetPoint("TOPRIGHT", -40, 0)
	search:SetJustifyH("LEFT")
	search:SetText(LUI:ColorToString(SEARCH, module:Color("Search")))
	editbox:SetAllPoints(search)
	
	-- Button to click Search
	local button = CreateFrame("Button", nil, LUIBags)
	button:EnableMouse(true)
	button:SetAllPoints(search)
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	button:SetScript("OnClick", function(self, button)
		if button == "RightButton" then
			OpenEditbox(self)
		else
			if self:GetParent().editbox:IsShown() then
				HideEditbox(self)
			end
		end
	end)
	
	-- Button tooltip
	button:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:ClearLines()
		GameTooltip:SetText(L["Bags_SearchHint"])
	end)
	button:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)
	button:SetScript("OnMouseDown", Bags_StartMoving)
	button:SetScript("OnMouseUp", Bags_StopMoving)
	
	LUIBags.editbox = editbox
	LUIBags.search = search
	LUIBags.searchButton = button
end

function module:CreateTitleBar()
	-- Close Button
	local closeBtn = CreateFrame("Button", "LUIBags_CloseButton", LUIBags, "UIPanelCloseButton")
	closeBtn:SetSize(32, 32)
	closeBtn:SetPoint("TOPRIGHT", -3, -3)
	closeBtn:RegisterForClicks("AnyUp")
	closeBtn:SetScript("OnClick", module.CloseBags)

	--TODO: Possibly change those two to use LUI FontStrings api
	-- Gold display, next to close button.
	local gold = LUIBags:CreateFontString(nil, "ARTWORK", "GameFontHighlightLarge")
	gold:SetJustifyH("RIGHT")
	gold:SetPoint("RIGHT", closeBtn, "LEFT", -3, 0)
	
	-- Watched Currency Display, next to gold
	local currency = LUIBags:CreateFontString(nil, "ARTWORK", "GameFontHighlightLarge")
	currency:SetJustifyH("RIGHT")
	currency:SetPoint("RIGHT", gold, "LEFT", -9, 0)
	currency:SetText(module:GetCurrencyString())
	
	--Hooking this function allows to update watched currencies without a ReloadUI
	module:SecureHook("BackpackTokenFrame_Update", "UpdateCurrencies")
	LUIBags:SetScript("OnEvent", module.UpdateCurrencies)
	LUIBags:RegisterEvent("PLAYER_MONEY")
	LUIBags:RegisterEvent("PLAYER_LOGIN")
	LUIBags:RegisterEvent("PLAYER_TRADE_MONEY")
	LUIBags:RegisterEvent("TRADE_MONEY_CHANGED")
	LUIBags:RegisterEvent("CURRENCY_DISPLAY_UPDATE")
	LUIBags:RegisterEvent("PLAYER_TRADE_CURRENCY")
	LUIBags:RegisterEvent("TRADE_CURRENCY_CHANGED")
	
	LUIBags.closeButton = closeBtn
	LUIBags.gold = gold
	LUIBags.currency = currency
end

function module:SetBags()
	if LUIBags then return end -- Bags already setup
	
	--Create the frame
	local frame = CreateFrame("Frame", "LUIBags", UIParent)
	frame:SetFrameStrata("HIGH")
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetToplevel(true)
	frame:SetClampedToScreen(true)
	LUIBags = frame

	frame:SetScript("OnShow", Bags_OnShow)
	frame:SetScript("OnHide", Bags_OnHide)
	frame:SetScript("OnMouseDown", Bags_StartMoving)
	frame:SetScript("OnMouseUp", Bags_StopMoving)
	
	module:CreateSearchEditBox()
	module:CreateTitleBar()
	
	-- Create BagsBar
	local bagsBar = CreateFrame("Frame", "LUIBags_BagsBar", frame)
	bagsBar:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", 0, 2)
	bagsBar:SetClampedToScreen(true)
	frame.BagsBar = bagsBar
	
	for i = 1, NUM_BAG_IDS do
		local id = BAG_ID_TABLE[i]
		if id <= 3 then -- BagsSlots uses a 0-3 system. Need Backpack Fix.
			local bagsSlot = module:NewBagsBarSlot(id)
			BagsBarSlots[#BagsBarSlots + 1] = bagsSlot
			bagsSlot:Show()
		end
		
		BagsInfo[id] = module:NewBagInfo(id)
		ItemSlots[id] = {}
	end
	
	module:SetBagFonts()
	module:SetBagsProperties()
	module:SetBagsDimensions()
	frame:Hide()
end

-- This function holds most properties that can be changed with options. To call whenever you change them.
function module:SetBagsProperties()
	local bagsBar = LUIBags.BagsBar
	
	-- Position
	local x, y = db.Bags.X, db.Bags.Y
	-- All Region coordinates API are based on the bottom left of the screen.
	LUIBags:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y)
	
	-- Backdrop
	local bagBackdrop = {
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = Media:Fetch("border", db.Bags.BorderTexture), 
		edgeSize = 5,
		insets = { left = 0, right = 0, top = 0, bottom = 0 }
	}
	LUIBags:SetBackdrop(bagBackdrop)
	--local bgr, bgg, bgb, bga = module:AlphaColor("Background")
	--local bdr, bdg, bdb, bda = module:AlphaColor("Border")
	--LUIBags:SetBackdropColor(bgr / 2, bgg / 2, bgb / 2, bga)
	--LUIBags:SetBackdropBorderColor(bdr * 1.5, bdg * 1.5, bdb * 1.5, bda)
	LUIBags:SetBackdropColor(module:AlphaColor("Background"))
	LUIBags:SetBackdropBorderColor(module:AlphaColor("Border"))
	bagsBar:SetBackdrop(bagBackdrop)
	bagsBar:SetBackdropColor(module:AlphaColor("Background"))
	bagsBar:SetBackdropBorderColor(module:AlphaColor("Border"))
	
	local itemBackdrop = {
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = Media:Fetch("border", db.Bags.BorderTexture),
		edgeSize = 15,
		insets = { left = 5, right = 5, top = 5, bottom = 5 },
	}
	
	for i = 1, NUM_BAG_IDS do
		local id = BAG_ID_TABLE[i]
		for i = 1, #ItemSlots[id] do
			local itemSlot = ItemSlots[id][i]
			itemSlot:SetBackdrop(itemBackdrop)
			itemSlot:SetBackdropColor(module:AlphaColor("Background"))
			itemSlot:SetBackdropBorderColor(module:AlphaColor("Border"))
		end
	end
end

function module:StyleIcon(frame)
	local iconTexture = _G[frame:GetName().."IconTexture"]
	iconTexture:SetTexCoord(0.08, 0.92, 0.08, 0.92)
	iconTexture:SetPoint("TOPLEFT", frame, 3, -3)
	iconTexture:SetPoint("BOTTOMRIGHT", frame, -3, 3)
	iconTexture:Show()
end

function module:NewBagInfo(id)
	if BagsInfo[id] then return BagsInfo[id] end
	local bagInfo = CreateFrame("Frame", "LUIBag"..id, LUIBags)
	bagInfo:SetID(id)
	return bagInfo
end

-- This function creates a new bag slot in the BagBar.
-- TODO: Flagged for Rewrite to support Backpack.
function module:NewBagsBarSlot(id)
	
	-- Check if there isn't already a BagsBarSlot for that id
	for i = 1, #BagsBarSlots do
		if BagsBarSlots[i].id == id then 
			return BagsBarSlots[i] 
		end
	end
	
	-- If nothing is found, make a new one.
	-- Note: The template needs to use Bag + ID + Slot format starting at the 10th character.
	local name = format("LUIBags__Bag%dSlot", id)
	local bagsSlot = CreateFrame("CheckButton", name, LUIBags.BagsBar, "BagSlotButtonTemplate")
	bagsSlot.id = id
	
	module:StyleIcon(bagsSlot)
	
	return bagsSlot
end

-- Creates a new item slot using Blizzard's template.
function module:NewItemSlot(id, slot)
	for i = 1, #ItemSlots[id] do
		if ItemSlots[id][i].bag == id and ItemSlots[id][i].slot == slot then
			return ItemSlots[id][i]
		end
	end
	
	local name = format("LUIBags_Item%s_%d", id, slot)
	local itemSlot = CreateFrame("Button", name, BagsInfo[id], "ContainerFrameItemButtonTemplate")
	itemSlot.bag = id
	itemSlot.slot = slot
	itemSlot:SetID(slot)
	itemSlot:Show()
	
	itemSlot:SetPushedTexture("")
	itemSlot:SetNormalTexture("")
	itemSlot:SetSize(32, 32)
	module:StyleIcon(itemSlot)
	
	itemSlot.cooldown = _G[itemSlot:GetName() .. "Cooldown"]
	itemSlot.cooldown:Show()
	
	itemSlot:SetBackdrop({
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = Media:Fetch("border", db.Bags.BorderTexture),
		edgeSize = 15,
		insets = { left = 5, right = 5, top = 5, bottom = 5 },
	})
	itemSlot:SetBackdropColor(module:AlphaColor("Background"))
	itemSlot:SetBackdropBorderColor(module:AlphaColor("Border"))
	
	return itemSlot
end

function module:SetBagFonts()
	LUIBags.gold:SetFont(Media:Fetch("font", db.Fonts.Bags.Name), db.Fonts.Bags.Size, db.Fonts.Bags.Flag)
	LUIBags.editbox:SetFont(Media:Fetch("font", db.Fonts.Bags.Name), db.Fonts.Bags.Size, db.Fonts.Bags.Flag)
	LUIBags.search:SetFont(Media:Fetch("font", db.Fonts.Bags.Name), db.Fonts.Bags.Size, db.Fonts.Bags.Flag)
	LUIBags.currency:SetFont(Media:Fetch("font", db.Fonts.Bags.Name), db.Fonts.Bags.Size, db.Fonts.Bags.Flag)
end

------------------------------------------------------
-- / MODULE FUNCTIONS / --
------------------------------------------------------
function module:IsProfessionBag(id)
	local _, bagType = GetContainerNumFreeSlots(id)
	if bagType and bagType > 0 then
		return true
	end
	return false
end

function module:GetCurrencyString()
	wipe(currencyString)
	for i = 1, MAX_WATCHED_TOKENS do
		local name, count, icon = GetBackpackCurrencyInfo(i)
		if name then
			currencyString[i] = format(CURRENCY_FORMAT,count,icon)
		end
	end
	return tconcat(currencyString, " ")
end

function module:UpdateCurrencies()
	LUIBags.gold:SetText(GetMoneyString(GetMoney()))
	LUIBags.currency:SetText(module:GetCurrencyString())
end


------------------------------------------------------
-- / BAG UPDATE FUNCTIONS / --
------------------------------------------------------
function module:SetBagsDimensions()
	local bagsBar = LUIBags.BagsBar
	
	local padding = db.Bags.Padding
	local spacing = db.Bags.Spacing
	local rowSize = db.Bags.RowSize
	
	--Readjust search anchor to account for padding.
	LUIBags.search:SetPoint("TOPLEFT", LUIBags, padding, -10)
	
	-- Still need to figure out this algorithm
	local width = 2 * padding + NUM_BAG_IDS * BAG_TEXTURE_SIZE + spacing * (NUM_BAG_IDS - 1)
	width = width - BAG_TEXTURE_SIZE - spacing -- Remove empty bag space until Backpack Fix
	bagsBar:SetHeight(2 * padding + BAG_TEXTURE_SIZE)
	bagsBar:SetWidth(width)

	for i = 1, #BagsBarSlots do
		-- We need a zero-based index, adjust accordingly.
		local xOffset = padding + (i - 1) * BAG_TEXTURE_SIZE + (i - 1) * spacing
		BagsBarSlots[i]:SetPoint("LEFT", bagsBar, "LEFT", xOffset, 0)
	end
	
	local slots = 0
	local rows = 0
	
	for i = 1, NUM_BAG_IDS do
		local id = BAG_ID_TABLE[i]
		local bagCount = GetContainerNumSlots(id)
		if bagCount > 0 then
			slots = slots + bagCount
		end
	end
	
	rows = floor(slots / rowSize)
	if (slots % rowSize) ~= 0 then
		rows = rows + 1
	end
	
	LUIBags:SetWidth(rowSize * 31 + spacing * (rowSize - 1) + padding * 2)
	LUIBags:SetHeight(rows * 31 + spacing * (rowSize - 1) + LAYOUT_OFFSET + padding * 2)
end

function module:SlotUpdate(itemSlot)
	local id, slot = itemSlot.bag, itemSlot.slot
	local texture, count, locked = GetContainerItemInfo(id, slot)
	
	-- Default Border when items are locked.
	if not itemSlot.lock then
		itemSlot:SetBackdropBorderColor(module:AlphaColor("Border"))
		-- Check for Profession Bag
		if module:IsProfessionBag(id) then
			itemSlot:SetBackdropBorderColor(module:AlphaColor("Professions"))
		end
	end
	
	-- Add the cooldown to our item slots.
	if itemSlot.cooldown then
		local start, duration, enable = GetContainerItemCooldown(id, slot)
		CooldownFrame_SetTimer(itemSlot.cooldown, start, duration, enable)
	end
	
	-- Show the "New Item Texture" if the item is new.
	-- Note: PTR added has them attached to the itemSlot.
	local newItemTexture, battlePayTexture
	if LUI.PTR then
		newItemTexture = itemSlot.NewItemTexture
		battlePayTexture = itemSlot.BattlepayItemTexture
	else
		newItemTexture = _G[itemSlot:GetName() .. "NewItemTexture"]
	end
	
	-- I do not know what BattlePay is either. This is based on the FrameXML.
	if C_NewItems.IsNewItem(id, slot) then
		if LUI.PTR and IsBattlePayItem(id, slot) then
			newItemTexture:Hide()
			battlePayTexture:Show()
		else
			if LUI.PTR then battlePayTexture:Hide() end
			newItemTexture:Show()
		end
	else
		if LUI.PTR then battlePayTexture:Hide() end
		newItemTexture:Hide()
	end
	
	
	-- Color Border according to quality
	local itemLink = GetContainerItemLink(id, slot)
	if itemLink then
		local name, _, quality = GetItemInfo(itemLink)
		if db.Bags.ItemQuality and quality and quality > 1 and not itemSlot.lock then
			itemSlot:SetBackdropBorderColor(GetItemQualityColor(quality))
		end
	end
	
	SetItemButtonTexture(itemSlot, texture)
	SetItemButtonCount(itemSlot, count)
	SetItemButtonDesaturated(itemSlot, locked, 0.5, 0.5, 0.5)
	
	itemSlot:Show()
end

function module:BagLayout()
	-- do they have to be reset?
	local slots = 0
	local rows = 0
	
	local rowSize = db.Bags.RowSize
	local padding = db.Bags.Padding
	local spacing = db.Bags.Spacing
	
	--note: color is Border Color, bgcolor is Background Color.
	
	module:UpdateCurrencies()

	-- Fill the bags.
	local index = 0
	for i = 1, NUM_BAG_IDS do
		local id = BAG_ID_TABLE[i]
		local bagCount = GetContainerNumSlots(id)
		
		if bagCount > 0 then
			BagsInfo[id]:Show()
			
			for i = 1, bagCount do
				ItemSlots[id][i] = module:NewItemSlot(id, i)
				local x = index % rowSize
				local y = floor(index / rowSize)
				
				local xOffset = padding + x * 31 + x * spacing
				local yOffset = LAYOUT_OFFSET + padding + y * 31 + (y - 1) * spacing
				ItemSlots[id][i]:ClearAllPoints()
				ItemSlots[id][i]:SetPoint("TOPLEFT", LUIBags, "TOPLEFT", xOffset, -yOffset)
				
				module:SlotUpdate(ItemSlots[id][i])
				index = index + 1
			end
		end
		
		-- If there are more itemslots than bagCount, hide those.
		LUI:Print(id, #ItemSlots[id], bagCount)
		for i = bagCount + 1, #ItemSlots[id] do
			if ItemSlots[id][i] then
				ItemSlots[id][i]:Hide()
			end
		end
	end
	
	module:SetBagsDimensions()
	LUIBags:SetScale(db.Bags.Scale)
	LUIBags.BagsBar:SetScale(db.Bags.Scale)
end
------------------------------------------------------
-- / EVENT HANDLING FUNCTIONS / --
------------------------------------------------------

function module:BagSlotUpdate(idList)
	if not ItemSlots then return end
	
	-- HACK: Whenever a bag is changed, BAG_UPDATE 0 triggers. (instead of the missing bag id)
	-- idList 0 is check individually here since pairs does not guarentee ordering.
	if idList[0] then
		module:BagLayout()
		return
	end
	
	for id in pairs(idList) do
		-- Check to see if we need to update the bags completly.
		if not ItemSlots[id] then
			module:BagLayout()
			return
		end
		for i = 1, #ItemSlots[id] do
			module:SlotUpdate(ItemSlots[id][i])
		end
	end
	module:SetBagsDimensions()
end

function module:ItemLockUpdate(event, id, slot)
	if not slot then return end
	module:SlotUpdate(ItemSlots[id][slot])
end

------------------------------------------------------
-- / FRAMEWORK FUNCTIONS / --
------------------------------------------------------
module.enableButton = true

function module:OnInitialize()
	LUI:RegisterModule(module)
	db = module:GetDB()
end

function module:OnEnable()
	--Add LUIBags to the "Can be closed using ESC" table
	tinsert(UISpecialFrames, "LUIBags")
	
	-- close bags before Enabling/Disabling the module
	CloseAllBags()
	
	--module:SecureHook("ToggleBackpack", "ToggleBags", true)
	--module:SecureHook("OpenAllBags",    "ToggleBags", true)
	--module:SecureHook("ToggleAllBags",  "ToggleBags", true)
	module:SecureHook("OpenBackpack",   "OpenBags",   true)
	module:SecureHook("CloseBackpack",  "CloseBags",  true)
	--module:SecureHook("CloseAllBags",   "CloseBags",  true)
	
	module:SetBags()
end

function module:OnDisable()
	CloseAllBags()
end