------------------------------------------------------
-- / SETUP AND LOCALS / --
------------------------------------------------------
local addonname, LUI = ...
local module = LUI:GetModule("Bags")
local Media = LibStub("LibSharedMedia-3.0")

local CLEANUP_TEXTURE_ATLAS = "bags-button-autosort-up"
local CLEANUP_TEXTURE = "Interface\\ContainerFrame\\Bags"
local CLEANUP_SOUND = "UI_BagSorting_01"

------------------------------------------------------
-- / BagBarSlotButtonTemplate / --
------------------------------------------------------

-- The end goal should be an identical button for Bags and Bank bars, but they use different APIs.
-- Note: Probably good idea to replace button with bagsSlot
-- TODO: Clean up and make more uniform, stop relying on Blizzard API.
function module:BagBarSlotButtonTemplate(index, id, name, parent)
	
	local button = module:CreateSlot(name, parent)
	button.isBag = 1 -- Blizzard API support
	button.id = id
	button.index = index
	button.container = parent:GetParent().name
	
	button:RegisterForDrag("LeftButton")
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	
	button:SetScript("OnClick", function(self) PutItemInBag(self.inventoryID) end)
	button:SetScript("OnLeave", function()
		GameTooltip:Hide()
		ResetCursor()
	end)
	
	--Try to have as few type-specific settings as possible
	if button.container == "Bags" then
	
		--PaperDollItemSlotButton OnLoad
		--slotName uses id - 1 due to Bag1-4 are refered as Bag0-Bag3
		local slotName = format("Bag%dSlot", index)
		local inventoryID, textureName = GetInventorySlotInfo(slotName)
		button.inventoryID = inventoryID
		button:SetID(inventoryID)
		
		local texture = _G[name.."IconTexture"]
		texture:SetTexture(textureName)
		button.backgroundTextureName = texturename
		
		--Rest of BagSlotTemplate OnLoad
		button:RegisterEvent("BAG_UPDATE_DELAYED")
		--button:RegisterEvent("INVENTORY_SEARCH_UPDATE")
		
		button.UpdateTooltip = BagSlotButton_OnEnter
		button.IconBorder:SetTexture("")
		button.IconBorder:SetSize(1,1)
		
		--TODO: Remove PaperDoll calls
		--BagSlotTemplate other events, unchecked. 
		button:SetScript("OnEvent", function(self, event, ...)
			if event == "BAG_UPDATE_DELAYED" then
				PaperDollItemSlotButton_Update(self)
				local quality = GetInventoryItemQuality("player", self.inventoryID)
				if quality and quality > 1 then
					self:SetBackdropBorderColor(GetItemQualityColor(quality))
				else
					self:SetBackdropBorderColor(module:AlphaColor("Border"))
				end
			else
				PaperDollItemSlotButton_OnEvent(self, event, ...)
			end
		end)
		-- OnShow/OnHide are just a bunch of update 
		button:SetScript("OnShow", PaperDollItemSlotButton_OnShow)
		button:SetScript("OnHide", PaperDollItemSlotButton_OnHide)
		button:SetScript("OnDragStart", function(self) PickupBagFromSlot(self.inventoryID) end)
		button:SetScript("OnReceiveDrag", function(self) PutItemInBag(self.inventoryID) end)
		button:SetScript("OnEnter", BagSlotButton_OnEnter)
	elseif button.container == "Bank" then	
		button:SetID(id-4)

		button.GetInventorySlot = ButtonInventorySlot;
		button.UpdateTooltip = BankFrameItemButton_OnEnter
		button.inventoryID = button:GetInventorySlot()
		
		button:SetScript("OnEvent", function(self, event, ...)
			if event == "BAG_UPDATE_DELAYED" then
				module:BankBagButtonUpdate(self)
			end
			-- Triggers when purchasing bank slots
			if event == "PLAYERBANKBAGSLOTS_CHANGED" then
				LUIBank:Layout()
			end
		end)
		button:SetScript("OnDragStart", BankFrameItemButtonBag_Pickup)
		button:SetScript("OnReceiveDrag", BankFrameItemButtonBag_OnClick)
		button:SetScript("OnEnter", BankFrameItemButton_OnEnter)
		
		button:SetScript("OnShow", function(self)
			module:BankBagButtonUpdate(self)
		end)
		
		--BankFrameItemButton_Update(button)
		module:BankBagButtonUpdate(button)
		BankFrameItemButton_UpdateLocked(button)
		button.tooltipText = button.tooltipText or ""
	end
	
	return button
end

function module:BankBagButtonUpdate(button)
	local texture = button.icon
	local textureName = GetInventoryItemTexture("player", button.inventoryID)
	local id, slotTextureName = GetInventorySlotInfo("Bag"..button.id)
	
	if textureName then
		texture:SetTexture(textureName)
	elseif slotTextureName then
		--If no bag texture is found, show empty slot. 
		texture:SetTexture(slotTextureName)
	end
	
	--Color border by quality
	local quality = GetInventoryItemQuality("player", button.inventoryID)
	if quality and quality > 1 then
		button:SetBackdropBorderColor(GetItemQualityColor(quality))
	else
		button:SetBackdropBorderColor(module:AlphaColor("Border"))
	end
	
	texture:Show()
end

------------------------------------------------------
-- / CleanUp Button / --
------------------------------------------------------

function module:CreateCleanUpButton(name, parent, sortFunc)
	local button = module:CreateSlot(name, parent)
	module:ApplyBackdrop(button, module.itemBackdrop)
	button:SetScript("OnClick", function()
			PlaySound(CLEANUP_SOUND)
			sortFunc()
		end)
	button:SetScript("OnEnter", function()
			GameTooltip:SetOwner(button)
			GameTooltip:SetText(BAG_CLEANUP_BAGS)
			GameTooltip:Show()
		end)
	button:SetScript("OnLeave", GameTooltip_Hide)
	
	--Adjust the icon to fit CleanUp.
	button.icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
	button.icon:SetTexture(CLEANUP_TEXTURE)
	button.icon:SetAtlas(CLEANUP_TEXTURE_ATLAS)
	
	return button
end