-- This module handles the colors section and themes
-- Also, possibly the CUSTOM_CLASS_COLOR implementation

-- Addon building reference.
local addonname, LUI = ...
local module = LUI:NewModule("Colors")
local L = LUI.L
local db

-- constants
local SANCTUARY = SANCTUARY_TERRITORY:sub(2, -2)  -- Remove parenthesis.
local FACTION_ALLIANCE = FACTION_ALLIANCE
local FACTION_HORDE = FACTION_HORDE
local MISCELLANEOUS = MISCELLANEOUS
local COLORS = COLORS

local STANDING_HATED      = FACTION_STANDING_LABEL1
local STANDING_HOSTILE    = FACTION_STANDING_LABEL2
local STANDING_UNFRIENDLY = FACTION_STANDING_LABEL3
local STANDING_NEUTRAL    = FACTION_STANDING_LABEL4
local STANDING_FRIENDLY   = FACTION_STANDING_LABEL5
local STANDING_HONORED    = FACTION_STANDING_LABEL6
local STANDING_REVERED    = FACTION_STANDING_LABEL7
local STANDING_EXALTED    = FACTION_STANDING_LABEL8

--Defaults
module.defaults = {
	profile = {
		Colors = {
			-- Class Colors
			DEATHKNIGHT = { r = 0.8,  g = 0.1,  b = 0.1,  },
			DRUID =       { r = 1,    g = 0.44, b = 0.15, },
			HUNTER =      { r = 0.22, g = 0.91, b = 0.18, },
			MAGE =        { r = 0.12, g = 0.58, b = 0.89, },
			MONK =        { r = 0.00, g = 1.00, b = 0.59, },
			PALADIN =     { r = 0.96, g = 0.21, b = 0.73, },
			PRIEST =      { r = 0.9,  g = 0.9,  b = 0.9,  },
			ROGUE =       { r = 0.95, g = 0.86, b = 0.16, },
			SHAMAN =      { r = 0.04, g = 0.39, b = 0.98, },
			WARLOCK =     { r = 0.57, g = 0.22, b = 1,    },
			WARRIOR =     { r = 1,    g = 0.78, b = 0.55, },

			-- Faction Colors
			Alliance =  { r = 0, g = 0.6, b = 1,   },
			Horde =     { r = 1, g = 0.3, b = 0.3, },
			Sanctuary = { r = 0, g = 1,   b = 1,   },

			-- Reaction Colors
			Hated =      { r = 1,   g = 0.3, b = 0.3, },
			Hostile =    { r = 1,   g = 0.3, b = 0.3, },
			Unfriendly = { r = 0.9, g = 0.2, b = 0,   },
			Neutral =    { r = 0.9, g = 0.7, b = 0,   },
			Friendly =   { r = 0,   g = 0.6, b = 0.1, },
			Honored =    { r = 0,   g = 0.6, b = 0.1, },
			Revered =    { r = 0,   g = 0.6, b = 0.1, },
			Exalted =    { r = 0,   g = 0.6, b = 0.1, },

			--Gradient
			Good =   { r = 0,   g = 1,   b = 0,   },
			Medium = { r = 1,   g = 1,   b = 0,   },
			Bad =    { r = 0.8, g = 0.3, b = 0.2, },

			--Level Differences
			DiffSkull = { r = 0.69, g = 0.31, b = 0.31, }, -- Target Level >= 5
			DiffHard =  { r = 0.71, g = 0.43, b = 0.27, }, -- Target Level >= 3
			DiffEqual = { r = 0.84, g = 0.75, b = 0.65, }, -- Target Level <> 2
			DiffEasy =  { r = 0.33, g = 0.59, b = 0.33, }, -- Target Level GreenQuestRange
			DiffLow =   { r = 0.55, g = 0.57, b = 0.61, }, -- Low Level Target
		},
	},
}
--[[
	LUI Colors
	DEATHKNIGHT = {0.8, 0.1, 0.1},
	DRUID = {1, 0.44, 0.15},
	HUNTER = {0.22, 0.91, 0.18},
	MAGE = {0.12, 0.58, 0.89},wqd
	MONK = {0.00, 1.00, 0.59},
	PRIEST = {0.9, 0.9, 0.9},
	PALADIN = {0.96, 0.21, 0.73},
	SHAMAN = {0.04, 0.39, 0.98},
	WARLOCK = {0.57, 0.22, 1},
	WARRIOR = {1, 0.78, 0.55},

	Blizzard Colors
	DEATHKNIGHT = {0.77, 0.12, 0.23},
	DRUID = {1.00, 0.49, 0.04},
	HUNTER = {0.67, 0.83, 0.45},
	MAGE = {0.41, 0.80, 0.94},
	MONK = {0.00, 1.00, 0.59},
	PRIEST = {1.00, 1.00, 1.00},
	ROGUE = {1.00, 0.96, 0.41}
	PALADIN = {0.96, 0.55, 0.73},
	SHAMAN = {0.00, 0.44, 0.87},
	WARLOCK = {0.58, 0.51, 0.79},
	WARRIOR = {0.78, 0.61, 0.43},

	LevelDiff = {
		[1] = {0.69, 0.31, 0.31}, -- Target Level >= 5
		[2] = {0.71, 0.43, 0.27}, -- Target Level >= 3
		[3] = {0.84, 0.75, 0.65}, -- Target Level <> 2
		[4] = {0.33, 0.59, 0.33}, -- Target Level GreenQuestRange
		[5] = {0.55, 0.57, 0.61}, -- Low Level Target
	},

	-- LEVEL DIFFS FUNCTIONS
	oUF:
	function(unit)
		local r, g, b
		local level = UnitLevel(unit)
		if level < 1 then
			r, g, b = unpack(module.colors.leveldiff[1])
		else
			local difference = level - UnitLevel("player")
			if difference >= 5 then
				r, g, b = unpack(module.colors.leveldiff[1])
			elseif difference >= 3 then
				r, g, b = unpack(module.colors.leveldiff[2])
			elseif difference >= -2 then
				r, g, b = unpack(module.colors.leveldiff[3])
			elseif -difference <= GetQuestGreenRange() then
				r, g, b = unpack(module.colors.leveldiff[4])
			else
				r, g, b = unpack(module.colors.leveldiff[5])
			end
		end
		return string.format("|cff%02x%02x%02x", r * 255, g * 255, b * 255)
	end

	Blizzard:
	function GetRelativeDifficultyColor(unitLevel, challengeLevel)
		local levelDiff = challengeLevel - unitLevel;
		local color;
		if ( levelDiff >= 5 ) then
			return QuestDifficultyColors["impossible"];
		elseif ( levelDiff >= 3 ) then
			return QuestDifficultyColors["verydifficult"];
		elseif ( levelDiff >= -2 ) then
			return QuestDifficultyColors["difficult"];
		elseif ( -levelDiff <= GetQuestGreenRange() ) then
			return QuestDifficultyColors["standard"];
		else
			return QuestDifficultyColors["trivial"];
		end
	end
--]]
------------------------------------------------------
-- / FRAMEWORK FUNCTIONS / --
------------------------------------------------------

module.order = 3 -- Makes Colors parts of the "core" options at the top.

function module:LoadOptions()

	--May be moved to the API if we need to.
	--List of localizedclass by englishClass
	local classL = {}
	for i = 1, 11 do
		local localizedClass, englishClass = GetClassInfo(i)
		classL[englishClass] = localizedClass
	end

	local options = {
			Header = module:NewHeader(COLORS, 1),
			Class = module:NewGroup(L["Colors_Classes"], 3, nil, nil, {
				DEATHKNIGHT = module:NewColor(classL["DEATHKNIGHT"], nil, 1),
				DRUID = module:NewColor(classL["DRUID"], nil, 2),
				HUNTER = module:NewColor(classL["HUNTER"], nil, 3),
				MAGE = module:NewColor(classL["MAGE"], nil, 4),
				MONK = module:NewColor(classL["MONK"], nil, 5),
				PALADIN = module:NewColor(classL["PALADIN"], nil, 6),
				PRIEST = module:NewColor(classL["PRIEST"], nil, 7),
				ROGUE = module:NewColor(classL["ROGUE"], nil, 8),
				SHAMAN = module:NewColor(classL["SHAMAN"], nil, 9),
				WARLOCK = module:NewColor(classL["WARLOCK"], nil, 10),
				WARRIOR = module:NewColor(classL["WARRIOR"], nil, 11),
			}),
			Factions = module:NewGroup(L["Colors_Factions"], 4, nil, nil, {
				Alliance = module:NewColor(FACTION_ALLIANCE, nil, 1),
				Horde = module:NewColor(FACTION_HORDE, nil, 2),
				Sanctuary = module:NewColor(SANCTUARY, nil, 3),
				Break = module:NewLineBreak(4),
				Hated = module:NewColor(STANDING_HATED, nil, 5),
				Hostile = module:NewColor(STANDING_HOSTILE, nil, 6),
				Unfriendly = module:NewColor(STANDING_UNFRIENDLY, nil, 7),
				Neutral = module:NewColor(STANDING_NEUTRAL, nil, 8),
				Friendly = module:NewColor(STANDING_FRIENDLY, nil, 9),
				Honored = module:NewColor(STANDING_HONORED, nil, 10),
				Revered = module:NewColor(STANDING_REVERED, nil, 11),
				Exalted = module:NewColor(STANDING_EXALTED, nil, 12),
			}),
			Misc = module:NewGroup(MISCELLANEOUS, 5, nil, nil, {
				GradientHeader = module:NewHeader(L["Colors_Gradients"], 1),
				Good = module:NewColor(L["Colors_Good"], nil, 2),
				Medium = module:NewColor(L["Colors_Medium"], nil, 3),
				Bad = module:NewColor(L["Colors_Bad"], nil, 4),
				-- Need much better names for these.
				LevelHeader = module:NewHeader(L["Color_Levels"], 5),
				DiffSkull = module:NewColor(L["Color_DiffSkull"], nil, 6, nil, nil, "full"),
				DiffHard = module:NewColor(L["Color_DiffHard"], nil, 7, nil, nil, "full"),
				DiffEqual = module:NewColor(L["Color_DiffEqual"], nil, 8, nil, nil, "full"),
				DiffEasy = module:NewColor(L["Color_DiffEasy"], nil, 9, nil, nil, "full"),
				DiffLow = module:NewColor(L["Color_DiffLow"], nil, 10, nil, nil, "full"),
			}),
		}
	return options
end

function module:OnInitialize()
	LUI:RegisterModule(module)
end

function module:OnEnable()
	db = module:GetDB()
end